@@ -16,11 +16,7 @@
 #' @return List of frames.
 #' @author Jakob Schwalb-Willmann
 #'
-#' @importFrom ggplot2 geom_line geom_text aes_string
-#' @importFrom geosphere distGeo
-#' 
-#' @importFrom grid arrow unit
-#' @importFrom ggplot2 expr
+#' @importFrom ggplot2 geom_line geom_text aes_string expr
 #'
 #' @examples 
 #' library(moveVis)
@@ -84,7 +80,7 @@ add_northarrow <- function(frames, height = 0.05, position = "bottomright", x =
   text.margin <- (max(arrow.data$y) - min(arrow.data$y))*label_margin
   text.data <- data.frame(x = arrow.data$x[1], y = min(arrow.data$y)-text.margin, label = label_text)
   
-  add_gg(frames, gg = expr(list(geom_line(aes_string(x = "x", y = "y"), data = arrow.data, arrow=arrow(length = unit(3.7, "mm")), size = size, colour = colour),
+  add_gg(frames, gg = expr(list(geom_line(aes_string(x = "x", y = "y"), data = arrow.data, arrow=grid::arrow(length = grid::unit(3.7, "mm")), size = size, colour = colour),
                                 geom_text(aes_string(x = "x", y = "y", label = "label"), text.data, colour = colour, size = label_size))),
          arrow.data = arrow.data, size = size, colour = colour, text.data = text.data, label_size = label_size)
 }
