@@ -3,8 +3,8 @@ context("settings")
 
 test_that("use_multicore", {
   expect_output(use_multicore(verbose = T))
-  expect_output(use_multicore(n_cores = 2, verbose = T))
-  expect_equal(getOption("moveVis.n_cores"), 2)
+  expect_output(use_multicore(n_cores = parallel::detectCores(), verbose = T))
+  expect_equal(getOption("moveVis.n_cores"), parallel::detectCores())
   expect_output(use_multicore(n_cores = 100, verbose = T))
   expect_equal(getOption("moveVis.n_cores"), parallel::detectCores())
   expect_output(use_multicore(n_cores = -1, verbose = T))
@@ -20,7 +20,3 @@ test_that("use_disk", {
   expect_output(expect_warning(use_disk(frames_to_disk = TRUE, n_memory_frames = 2)))
   use_disk(frames_to_disk = FALSE, verbose = F)
 })
-
-test_that("onLoad", {
-  expect_invisible(moveVis:::.onLoad())
-})
\ No newline at end of file
