@@ -2,11 +2,15 @@ skip_on_cran()
 context("view_spatial")
 
 test_that("view_spatial", {
+  
   # correct calls
-  expect_is(view_spatial(m), "mapview")
-  expect_is(view_spatial(m, render_as = "leaflet"), "leaflet")
-  expect_is(view_spatial(m, time_labels = FALSE, path_legend = FALSE), "mapview")
-  expect_is(view_spatial(m[[1]], time_labels = FALSE, path_legend = FALSE), "mapview")
+  if(isTRUE(check_mapview)){
+    expect_is(view_spatial(m), "mapview")
+    expect_is(view_spatial(m, time_labels = FALSE, path_legend = FALSE), "mapview")
+    expect_is(view_spatial(m[[1]], time_labels = FALSE, path_legend = FALSE), "mapview")
+  }else expect_error(view_spatial(m))
+  
+  if(isTRUE(check_leaflet)) expect_is(view_spatial(m, render_as = "leaflet"), "leaflet") else expect_error(view_spatial(m))
   
   # false calls
   expect_error(view_spatial(m, render_as = "abc"))
